\ A Test Framework in Standard Forth
\ 
\ Copyright (C) 2007 Joshua I. Grams
\ May be used, modified, and redistributed without limitation.

require modules.fs

module tester


\ Syntax:
	\ { test code => correct results }
	\ Checks stack (and heap if correct-results uses it).


\ Stack Checking

variable stack-depth
create stack 16 cells allot

: save-stack ( ... -- )
	depth dup stack-depth !  0 ?do stack i cells + ! loop ;

: load-stack ( u -- ... )
	dup if 1- 0 swap do stack i cells + @ -1 +loop else drop then ;

: empty-stack ( ... -- )  depth 0 ?do drop loop ;

: .stacks
	." Expected: " .s empty-stack
	stack-depth @ load-stack ." Got: " .s empty-stack ;

: check-stack ( ... -- )
	depth stack-depth @ <> if .stacks abort then  
	depth 0 ?do
		dup stack i cells + @ <> if
			i load-stack .stacks abort
		then drop
	loop ;


\ Heap Checking

variable data
variable answers

: data-length ( -- u )  answers @ data @ - ;

: answer-length ( -- u )  here answers @ - ;

: .heaps
	." Expected: " answers @ answer-length dump
	." Got: " data @ data-length dump cr ;

: check-heap
	data-length answer-length <> if .heaps abort then
	data-length 0 do
		data @ i + c@  answers @ i + c@ <> if
			.heaps abort
		then
	loop ;

: ?check-heap  here answers @ <> if check-heap then ;


\ The Public Interface

public:

: { here data ! ;
: => save-stack  here answers ! ;
: } check-stack ?check-heap ;

end-module
